angular.module('paperworkNotes').controller('SidebarManageNotebooksController',
  function($scope, $rootScope, $location, $routeParams, NotebooksService) {
    $scope.modalList = [];

    $('#modalManageNotebooks').on('hidden.bs.modal', function(e) {
      //$scope.modalList = [];
      //$scope.$apply();    // Because not angular scope
    });

    //$('#modalManageNotebooks').on('show.bs.modal', function (e) {
    // Added watch to rebuild list when we add new element
    $rootScope.$watch('notebooks', function(newValue, oldValue) {
      var data = ($.isArray($rootScope.notebooks)) ? $rootScope.notebooks.slice() : [];
      for(var i in data) {
        data[i].css_class = "notebook_line";
        if(data[i].type == 2) {
          data.splice(i, 1);
          //break;
        }
        if(data[i].type == 1) {
            data[i].css_class = "collection_line";
        }
        for(var j in data[i].children) {
          data.push(data[i].children[j]);
        }
      }
      $scope.modalList = data;
    });

    var buildShortcutChk = function(editable, newRow) {
      var shortcut = false;
      if(!newRow) {
        shortcut = !!NotebooksService.getShortcutByNotebookIdLocal(editable.options.pk);
      }

      editable.container.$form.find('.control-group').after(
        $('<div/>').addClass('checkbox shortcut-chk').append(
          $('<label/>').append(
            $('<input/>').attr({
              'type':  'checkbox',
              'name':  'shortcut',
              'value': '1'
            }).prop('checked', shortcut)
          ).append($rootScope.i18n.notebooks.add_shortcut)
        )
      );
    };

    var parseParams = function(params) {
      return {
        id:       params.pk,
        title:    params.value,
        shortcut: $(this).parent().find("input[name='shortcut']").is(':checked'),
        type:     0
      };
    };

    $scope.$on('ngRepeatFinished', function(ngRepeatFinishedEvent) {
      $('#modalManageNotebooks .notebook_line').editable({
        params:       parseParams,
        send:         'always',
        savenochange: true,
        url:          function(data) {
          NotebooksService.updateNotebook(data.id, data, function(status, responseData) {
            switch(status) {
              case 200:
                NotebooksService.getNotebooks();
                NotebooksService.getNotebookShortcuts(null);
                break;
              case 400:
                for(var i in responseData.errors) {
                  var msg = responseData.errors[i];
                  break;
                }
                $('#modalManageNotebooks').find(".line[data-pk='" + data.id + "']").tooltip({
                  title:   msg,
                  trigger: 'manual'
                }).tooltip('show');
                break;
            }
          });
        }
      }).on('shown', function(e, editable) {
        buildShortcutChk(editable, false);
        editable.$element.tooltip('destroy');
      });
    });

    $scope.removeFromCollection = function(id) {
      NotebooksService.removeNotebookFromCollection(id, function(status) {
        NotebooksService.getNotebooks();
        NotebooksService.getNotebookShortcuts();
      });
    };

    $scope.deleteItem = function(id, type) {
      var modalTitle, modalContent;
      if(type == 0) {
          modalTitle = $rootScope.i18n.keywords.delete_notebook_question;
          modalContent = $rootScope.i18n.keywords.delete_notebook_message;
      }else if(type == 1) {
          modalTitle = $rootScope.i18n.keywords.delete_collection_question;
          modalContent = $rootScope.i18n.keywords.delete_collection_message;
      }
      if(type == 1) {
          $rootScope.modalMessageBox = {
            'title':   modalTitle,
            'content': modalContent,
            'buttons': [
                {
                    'label':     $rootScope.i18n.keywords.cancel,
                    'isDismiss': true
                },
                {
                    'id':    'button-yes',
                    'label': $rootScope.i18n.keywords.yes,
                    'class': 'btn-warning',
                    'click': function() {
                      NotebooksService.deleteNotebook(id, function(status, data) {
                        NotebooksService.getNotebooks();
                        NotebooksService.getNotebookShortcuts();
                      });
                      $('#modalManageNotebooks').find(".line[data-pk='" + id + "']").closest('.row').remove();
                      return true;
                    }
                }
            ]
          };
          $('#modalMessageBox').modal('show');
      }else{
          $rootScope.modalNotebookDelete = {
              'action': false,
              'notebookId': id
          };
          $('#modalNotebookDelete').modal('show');
      }
    };

    $scope.addNotebook = function() {

      // build item
      var $row = $('<div/>').addClass('row')
        .append(
        $('<div/>').addClass('col-sm-10').append(
          $('<a/>').addClass('line').data('name', 'title')
        )
      )
        .append(
        $('<div/>').addClass('col-sm-2')
      );

      $row.find('a').editable({
        params: parseParams,
        url:    function(data) {
          NotebooksService.createNotebook(data, function(status, responseData) {
            switch(status) {
              case 200:
                NotebooksService.getNotebooks();
                NotebooksService.getNotebookShortcuts(null);

                $row.remove();
                //$scope.modalList.push(responseData.response);  --- we will  use $watch instead
                break;
              case 400:
                for(var i in responseData.errors) {
                  var msg = responseData.errors[i];
                  break;
                }
                $row.find(".line").tooltip({
                  title:   msg,
                  trigger: 'manual'
                }).tooltip('show');

                setTimeout(function() {
                  $row.remove();
                }, 3000);
                break;
            }
          });
        }
      }).on('hidden', function(e, reason) {
        if(reason != 'save') {
          $row.remove();
        }
      }).on('shown', function(e, editable) {
        buildShortcutChk(editable, true);
        editable.$element.tooltip('destroy');
      });

      $('#modalManageNotebooks .manage-list-content').append($row);

      setTimeout(function() { // doesn't work without timeout
        $row.find('a').editable('show');
      }, 1);
    };

    $scope.editLineCalled = function(id, type) {
        if(type == 1) {
            $rootScope.modalEditCollection(id);
        }
    };

  });
